# Description: MCP completions module for LogicMonitor tool arguments.
# Description: Provides auto-complete functionality for argument values.

from lm_mcp.completions.handlers import get_completions

__all__ = ["get_completions"]
